package com.hammurapi.review;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.hammurapi.config.ObjectDefinition;
import com.hammurapi.config.bootstrap.ConfigurationException;
import com.hammurapi.config.runtime.Component;
import com.hammurapi.config.runtime.ConfigurationContext;
import com.hammurapi.util.Context;

/**
 * Delegates method invocations to reporters found in context.
 * @author Pavel Vlasov
 *
 */
public class ContextReporter implements Reporter<Object>, Component<ObjectDefinition> {
	
	private Context context;
	
	@SuppressWarnings("unchecked")
	Iterator<Reporter> reporters() {
		return context.lookupAll(Reporter.class);
	}

	@SuppressWarnings("unchecked")
	@Override
	public void close() throws ReviewException {
		Iterator<Reporter> it = reporters();
		while (it.hasNext()) {
			it.next().close();
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public ObservationSink createObservationSink(Module module) throws ReviewException {
		final List<ObservationSink> observationSinks = new ArrayList<ObservationSink>();
		Iterator<Reporter> it = reporters();
		while (it.hasNext()) {
			observationSinks.add(it.next().createObservationSink(module));
		}
		return new ObservationSink() {
			
			@Override
			public void consumeObservation(Observation observation)	throws ReviewException {
				for (ObservationSink sink: observationSinks) {
					sink.consumeObservation(observation);
				}				
			}
			
			@Override
			public void close() throws ReviewException {
				for (ObservationSink sink: observationSinks) {
					sink.close();
				}				
			}
		};
	}

	@SuppressWarnings("unchecked")
	@Override
	public void onException(Exception e) {
		Iterator<Reporter> it = reporters();
		while (it.hasNext()) {
			it.next().onException(e);
		}
	}

	@Override
	public void init(ConfigurationContext<ObjectDefinition> context) throws ConfigurationException {
		this.context = context;		
	}

}
