package com.hammurapi.review;

import java.io.PrintStream;

public class PrintStreamReporter implements Reporter<Object> {
	
	private PrintStream out;
	private boolean closeStream;

	public PrintStreamReporter(PrintStream out, boolean closeStream) {
		this.out = out;
		this.closeStream = closeStream;
		out.println("New reporter.");
	}
	
	/**
	 * Constructs print stream reporter from System.out
	 */
	public PrintStreamReporter() {
		this(System.out, false);
	}

	@Override
	public void close() throws ReviewException {
		out.println("Review finished.");
		if (closeStream) {
			out.close();
		}		
	}

	@Override
	public ObservationSink createObservationSink(final Module module) throws ReviewException {
		out.println("Started review of module "+module.getName());
		
		return new ObservationSink() {
			
			@Override
			public void consumeObservation(Observation observation)	throws ReviewException {
				out.println("["+module.getName()+"] "+observation);
			}
			
			@Override
			public void close() throws ReviewException {
				out.println("Finished review of moudle "+module.getName());				
			}
		};
	}

	@Override
	public void onException(Exception e) {
		out.println("Exception during review: ");
		e.printStackTrace(out);		
	}

}
