package com.hammurapi.review;

/**
 * Reporters consume review observations.
 * @author Pavel Vlasov
 * @param <M> Model type.
 */
public interface Reporter<M> {

	/**
	 * Creates observation sink for a module.
	 * @param moduleName
	 * @param customConfig
	 * @param model
	 * @return
	 * @throws ReviewException
	 */
	ObservationSink createObservationSink(Module module) throws ReviewException;
	
	/**
	 * This method is invoked when exception is thrown before model and observation sinks are constructed.
	 * @param moduleName
	 * @param e
	 */
	void onException(Exception e);
	
	/**
	 * Closes reporter.
	 * @throws ReviewException
	 */
	void close() throws ReviewException;
}
