package com.hammurapi.review;

import java.util.logging.Logger;

import org.apache.commons.pool.impl.StackKeyedObjectPool;

import com.hammurapi.config.bootstrap.ConfigurationException;
import com.hammurapi.config.runtime.FactoryConfig;
import com.hammurapi.flow.Flow;
import com.hammurapi.reasoning.ForwardReasoningSessionFactory;
import com.hammurapi.reasoning.ReasoningException;
import com.hammurapi.reasoning.impl.ForwardReasoningSessionFactoryImpl;
import com.hammurapi.reasoning.impl.PoolingForwardReasoningSessionFactory;
import com.hammurapi.review.util.InspectorSetCompiler;
import com.hammurapi.util.SingletonChainingContext;

public class SessionPoolingReviewDriver extends ReviewDriverBase {
	
	private static final Logger logger = Logger.getLogger(SessionPoolingReviewDriver.class.getName());
	
	private int sessionPoolSize;

	public SessionPoolingReviewDriver(int sessionPoolSize) {
		this.sessionPoolSize = sessionPoolSize;
	}

	@Override
	protected ForwardReasoningSessionFactory<Object> createInspectorSetSessionFactory(InspectorSet inspectorSet, FactoryConfig moduleFactoryConfig) throws ConfigurationException, ReasoningException {
		logger.fine("Creating a pooling session factory of size "+sessionPoolSize+" for inspector set "+inspectorSet.getName());
		StackKeyedObjectPool pool = new StackKeyedObjectPool(sessionPoolSize);
		InspectorSetCompiler inspectorSetCompiler = new InspectorSetCompiler();
		Flow compiledInspectorSet = inspectorSetCompiler.compile(inspectorSet, moduleFactoryConfig); 
		FactoryConfig inspectorSetConfig;
		try {
			inspectorSetConfig = (FactoryConfig) moduleFactoryConfig.clone();
		} catch (CloneNotSupportedException e) {
			throw new ConfigurationException(e);
		}
		inspectorSetConfig.setContext(new SingletonChainingContext<InspectorSet>(InspectorSet.class, inspectorSet, moduleFactoryConfig.getContext()));
		ForwardReasoningSessionFactory<Object> master = new ForwardReasoningSessionFactoryImpl<Object>(compiledInspectorSet, inspectorSetConfig);
		return new PoolingForwardReasoningSessionFactory<Object>(master, pool);
	}
}
