package com.hammurapi.review.util;

import java.util.Iterator;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;

import com.hammurapi.convert.AtomicConverterBase;
import com.hammurapi.convert.Converter;
import com.hammurapi.util.Context;

/**
 * Converts EObject to iterable which includes the object itself into iteration.
 * @author Pavel Vlasov
 *
 */
public class EObjectToIterableConverter extends AtomicConverterBase<EObject, Iterable<EObject>> {
	
	@SuppressWarnings("unchecked")
	public EObjectToIterableConverter() {
		super(EObject.class, (Class<Iterable<EObject>>) (Class) Iterable.class);
	}	

	@Override
	public Iterable<EObject> convert(final EObject source, Converter master, Context context, ClassLoader classLoader) {
		return new Iterable<EObject>() {
			@Override
			public Iterator<EObject> iterator() {
				return new TreeIterator<EObject>() {
					private TreeIterator<EObject> master;
					private boolean pruned;

					@Override
					public void prune() {
						if (master==null) {
							pruned=true;
						} else {
							master.prune();
						}
					}

					@Override
					public boolean hasNext() {
						return master==null ? !pruned : master.hasNext();
					}

					@Override
					public EObject next() {
						if (master==null) {
							if (pruned) {
								throw new IllegalStateException();
							} else {
								master = source.eAllContents();
								return source;
							}
						}
						
						return master.next();
					}

					@Override
					public void remove() {
						if (master==null) {
							throw new IllegalStateException();
						} else {
							master.remove();
						}
					}
					
				};
			}
			
		};
	}

}
