package com.hammurapi.review.util;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ServiceLoader;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage.Registry;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.hammurapi.config.ConfigFactory;
import com.hammurapi.config.NamedObjectDefinition;
import com.hammurapi.config.runtime.PackageProvider;
import com.hammurapi.review.Inspector;
import com.hammurapi.review.InspectorCategory;
import com.hammurapi.review.InspectorSet;
import com.hammurapi.review.ReviewFactory;

/**
 * Migrates rule set definitions from 5.x to 6
 * @author Pavel Vlasov
 *
 */
public class Migrator {

	private static final String TYPE = "type";

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		System.out.println("Usage: java [options] "+Migrator.class.getName()+" <source file> <target file>");
		Document input = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new File(args[0]));		
		
		ResourceSet resourceSet = new ResourceSetImpl();		
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());

		Registry registry = resourceSet.getPackageRegistry();
		Iterator<PackageProvider> ppit = ServiceLoader.load(PackageProvider.class, Migrator.class.getClassLoader()).iterator();
        while (ppit.hasNext()) {
        	ppit.next().registerPackages(registry);
        }
		
        InspectorSet inspectorSet = ReviewFactory.eINSTANCE.createInspectorSet();
        Element root = input.getDocumentElement();
        XPath xPath = XPathFactory.newInstance().newXPath(); 
		inspectorSet.setName(xPath.evaluate("name/text()", root));
		inspectorSet.setDescription(xPath.evaluate("description/text()", root));
		
		Map<List<String>, InspectorCategory> categoryMap = new HashMap<List<String>, InspectorCategory>();
		NodeList inspectors = (NodeList) xPath.evaluate("rules/rule", root, XPathConstants.NODESET);
		for (int i=0, l=inspectors.getLength(); i<l; ++i) {
			Inspector inspector = ReviewFactory.eINSTANCE.createInspector();
			inspectorSet.getInspectors().add(inspector);
			Node iNode = inspectors.item(i);
			inspector.setName(xPath.evaluate("name/text()", iNode));
			inspector.setDescription(xPath.evaluate("description/text()", iNode));
			
			List<String> categories = new ArrayList<String>();
			NodeList cnl = (NodeList) xPath.evaluate("category", iNode, XPathConstants.NODESET);
			for (int j=0, cl=cnl.getLength(); j<cl; ++j) {
				categories.add(xPath.evaluate("text()", cnl.item(j)));
			}
			if (!categories.isEmpty()) {
				inspector.setCategory(getCategory(categories, categoryMap, inspectorSet));
			}
			inspector.setSeverity(Integer.parseInt(xPath.evaluate("severity/text()", iNode)));
			inspector.setType(((Element) iNode).getAttribute(TYPE));
			NodeList allChildren=iNode.getChildNodes();
			for (int j=0, cl=allChildren.getLength(); j<cl; ++j) {
				Node pNode = allChildren.item(j);
				if (pNode instanceof Element) {
					String pName = pNode.getNodeName();
					if (!"name".equals(pName) && !"severity".equals(pName) && !"description".equals(pName) && !"category".equals(pName)) {
						NamedObjectDefinition property = ConfigFactory.eINSTANCE.createNamedObjectDefinition();
						inspector.getProperty().add(property);
						property.setName(pName);
						Element pElement = (Element) pNode;
						if (pElement.hasAttribute(TYPE)) {
							property.setType(pElement.getAttribute(TYPE));
						}
						property.setValue(xPath.evaluate("text()", pNode));
					}											
				}
			}
		}
		
		Resource inspectorSetResource = resourceSet.createResource(URI.createFileURI(new File(args[1]).getAbsolutePath()));
		if (inspectorSetResource instanceof XMLResource) {
			((XMLResource) inspectorSetResource).setEncoding("UTF-8");
		}
		inspectorSetResource.getContents().add(inspectorSet);
		inspectorSetResource.save(null);
	}
	
	private static InspectorCategory getCategory(List<String> categories, Map<List<String>, InspectorCategory> categoryMap, InspectorSet inspectorSet) {
		InspectorCategory ret = categoryMap.get(categories);
		if (ret==null) {
			ret = ReviewFactory.eINSTANCE.createInspectorCategory();
			ret.setName(categories.get(categories.size()-1));
			if (categories.size()==1) {
				inspectorSet.getCategories().add(ret);
			} else {
				List<String> pc = new ArrayList<String>(categories);
				pc.remove(categories.size()-1);
				getCategory(pc, categoryMap, inspectorSet).getSubCategory().add(ret);
			}
			categoryMap.put(categories, ret);
		}
		return ret;
	}

}
